@@ -26,6 +26,7 @@
 #include <string.h>
 #include <unistd.h>
 
+#include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 #include <glade/glade.h>
 #include <gconf/gconf-client.h>
@@ -46,6 +47,7 @@
 #include "sj-util.h"
 #include "sj-prefs.h"
 #include "sj-play.h"
+#include "bacon-volume.h"
 
 gboolean on_delete_event (GtkWidget *widget, GdkEvent *event, gpointer user_data);
 
@@ -67,9 +69,10 @@ GtkWidget *main_window;
 static GtkWidget *title_entry, *artist_entry, *duration_label, *genre_entry;
 static GtkWidget *track_listview, *extract_button, *play_button;
 static GtkWidget *status_bar;
-static GtkWidget *extract_menuitem, *play_menuitem, *select_all_menuitem, *deselect_all_menuitem;
+static GtkWidget *extract_menuitem, *play_menuitem, *next_menuitem, *prev_menuitem, *select_all_menuitem, *deselect_all_menuitem;
 GtkListStore *track_store;
 static BaconMessageConnection *connection;
+GtkCellRenderer *toggle_renderer, *title_renderer, *artist_renderer;
 
 const char *base_uri, *path_pattern, *file_pattern;
 NautilusBurnDrive *drive = NULL;
@@ -89,6 +92,66 @@ gboolean autostart = FALSE, autoplay = FALSE;
 #define SOURCE_GLADE "../data/sound-juicer.glade"
 #define INSTALLED_GLADE DATADIR"/sound-juicer/sound-juicer.glade"
 
+static void
+add_stock_icon (GtkIconFactory *sj_icon_factory,
+                const gchar *stock_id,
+                GtkIconSize  size,
+                const gchar *file_name)
+{
+  GtkIconSource *source;
+  GtkIconSet    *set;
+  GdkPixbuf     *pixbuf;
+
+  source = gtk_icon_source_new ();
+
+  gtk_icon_source_set_size (source, size);
+  gtk_icon_source_set_size_wildcarded (source, FALSE);
+
+  pixbuf = gdk_pixbuf_new_from_file (file_name, NULL);
+
+  gtk_icon_source_set_pixbuf (source, pixbuf);
+
+  g_object_unref (pixbuf);
+
+  set = gtk_icon_set_new ();
+
+  gtk_icon_set_add_source (set, source);
+  gtk_icon_source_free (source);
+
+  gtk_icon_factory_add (sj_icon_factory, stock_id, set);
+
+  gtk_icon_set_unref (set);
+}
+
+void
+sj_stock_init (void)
+{
+  static gboolean initialized = FALSE;
+  static GtkIconFactory *sj_icon_factory = NULL;
+
+  static GtkStockItem sj_stock_items[] =
+  {
+    { SJ_STOCK_PLAYING, NULL, 0, 0, NULL },
+    { SJ_STOCK_RECORDING, NULL, 0, 0, NULL },
+    { SJ_STOCK_EXTRACT, "E_xtract", GDK_CONTROL_MASK, GDK_Return, NULL }
+  };
+
+  if (initialized)
+    return;
+
+  sj_icon_factory = gtk_icon_factory_new ();
+
+  add_stock_icon (sj_icon_factory, SJ_STOCK_PLAYING, GTK_ICON_SIZE_MENU, PKGDATADIR"/sj-play.png");
+  add_stock_icon (sj_icon_factory, SJ_STOCK_RECORDING, GTK_ICON_SIZE_MENU, PKGDATADIR"/sj-record.png");
+  gtk_icon_factory_add (sj_icon_factory, SJ_STOCK_EXTRACT, gtk_icon_factory_lookup_default (GTK_STOCK_CDROM));
+
+  gtk_icon_factory_add_default (sj_icon_factory);
+
+  gtk_stock_add_static (sj_stock_items, G_N_ELEMENTS (sj_stock_items));
+
+  initialized = TRUE;
+}
+
 void
 sj_main_set_title (const char* detail)
 {
@@ -156,6 +219,7 @@ gboolean poll_tray_opened (gpointer data)
   if (new_status != tray_opened && new_status == FALSE) {
     reread_cd (TRUE);
   } else if (new_status != tray_opened && new_status == TRUE) {
+    stop_ui_hack ();
     update_ui_for_album (NULL);
   }
   tray_opened = new_status;
@@ -241,12 +305,11 @@ static void duration_cell_data_cb (GtkTreeViewColumn *tree_column,
   g_free (string);
 }
 
-extern gint current_track;
-static void title_cell_icon_data_cb (GtkTreeViewColumn *tree_column,
-                                GtkCellRenderer *cell,
-                                GtkTreeModel *tree_model,
-                                GtkTreeIter *iter,
-                                gpointer data)
+static void number_cell_icon_data_cb (GtkTreeViewColumn *tree_column,
+				      GtkCellRenderer *cell,
+				      GtkTreeModel *tree_model,
+				      GtkTreeIter *iter,
+				      gpointer data)
 {
   TrackState state;
   gtk_tree_model_get (tree_model, iter, COLUMN_STATE, &state, -1);
@@ -255,10 +318,10 @@ static void title_cell_icon_data_cb (GtkTreeViewColumn *tree_column,
     g_object_set (G_OBJECT (cell), "stock-id", "", NULL);
     break;
   case STATE_PLAYING:
-    g_object_set (G_OBJECT (cell), "stock-id", GTK_STOCK_MEDIA_PLAY, NULL);
+    g_object_set (G_OBJECT (cell), "stock-id", SJ_STOCK_PLAYING, NULL);
     break;
   case STATE_EXTRACTING:
-    g_object_set (G_OBJECT (cell), "stock-id", GTK_STOCK_MEDIA_RECORD, NULL);
+    g_object_set (G_OBJECT (cell), "stock-id", SJ_STOCK_RECORDING, NULL);
     break;
   default:
     g_warning("Unhandled track state %d\n", state);
@@ -290,6 +353,8 @@ static void update_ui_for_album (AlbumDetails *album)
     gtk_widget_set_sensitive (extract_menuitem, FALSE);
     gtk_widget_set_sensitive (select_all_menuitem, FALSE);
     gtk_widget_set_sensitive (deselect_all_menuitem, TRUE);
+    gtk_widget_set_sensitive (prev_menuitem, FALSE);
+    gtk_widget_set_sensitive (next_menuitem, FALSE);
   } else {
     gtk_list_store_clear (track_store);
 
@@ -309,6 +374,8 @@ static void update_ui_for_album (AlbumDetails *album)
     gtk_widget_set_sensitive (extract_menuitem, TRUE);
     gtk_widget_set_sensitive (select_all_menuitem, FALSE);
     gtk_widget_set_sensitive (deselect_all_menuitem, TRUE);
+    gtk_widget_set_sensitive (prev_menuitem, FALSE);
+    gtk_widget_set_sensitive (next_menuitem, FALSE);
     
     for (l = album->tracks; l; l=g_list_next (l)) {
       GtkTreeIter iter;
@@ -979,6 +1046,15 @@ static GtkTreeModel* create_genre_list(void) {
   return GTK_TREE_MODEL (store);
 }
 
+GtkWidget *
+sj_make_volume_button (void)
+{
+  GtkWidget *w = bacon_volume_button_new (GTK_ICON_SIZE_SMALL_TOOLBAR,
+					  0.0, 1.0, 0.02);
+  bacon_volume_button_set_value (BACON_VOLUME_BUTTON (w), 1.0);
+  return w;
+}
+
 static void
 upgrade_gconf (void)
 {
@@ -1011,6 +1087,7 @@ on_message_received (const char *message, gpointer user_data)
 int main (int argc, char **argv)
 {
   GError *error = NULL;
+  GtkTreeSelection *selection;
   struct poptOption options[] = {
     { NULL, '\0', POPT_ARG_INCLUDE_TABLE, NULL, 0, "GStreamer", NULL },
     { "auto-start", 'a', POPT_ARG_NONE, &autostart, 0, "Start extracting immediately", NULL},
@@ -1031,6 +1108,8 @@ int main (int argc, char **argv)
                       NULL);
   g_set_application_name (_("Sound Juicer"));
 
+  sj_stock_init ();
+
   connection = bacon_message_connection_new ("sound-juicer");
   if (bacon_message_connection_get_is_server (connection) == FALSE) {
 	  bacon_message_connection_send (connection, RAISE_WINDOW);
@@ -1101,6 +1180,8 @@ int main (int argc, char **argv)
   extract_menuitem = glade_xml_get_widget (glade, "extract_menuitem");
   play_button = glade_xml_get_widget (glade, "play_button");
   play_menuitem = glade_xml_get_widget (glade, "play_menuitem");
+  next_menuitem = glade_xml_get_widget (glade, "next_track_menuitem");
+  prev_menuitem = glade_xml_get_widget (glade, "previous_track_menuitem");
   status_bar = glade_xml_get_widget (glade, "status_bar");
 
   {
@@ -1118,47 +1199,48 @@ int main (int argc, char **argv)
     GtkTreeViewColumn *column;
     GtkCellRenderer *renderer;
     
-    renderer = gtk_cell_renderer_toggle_new ();
-    g_signal_connect (renderer, "toggled", G_CALLBACK (on_extract_toggled), NULL);
-    column = gtk_tree_view_column_new_with_attributes (_("Extract"),
-                                                       renderer,
+    toggle_renderer = gtk_cell_renderer_toggle_new ();
+    g_signal_connect (toggle_renderer, "toggled", G_CALLBACK (on_extract_toggled), NULL);
+    column = gtk_tree_view_column_new_with_attributes ("",
+                                                       toggle_renderer,
                                                        "active", COLUMN_EXTRACT,
                                                        NULL);
     gtk_tree_view_column_set_resizable (column, TRUE);
     gtk_tree_view_append_column (GTK_TREE_VIEW (track_listview), column);
 
+    column = gtk_tree_view_column_new ();
+    gtk_tree_view_column_set_title (column, _("Track"));
+    gtk_tree_view_column_set_expand (column, FALSE);
+    gtk_tree_view_column_set_resizable (column, FALSE);
     renderer = gtk_cell_renderer_text_new ();
-    column = gtk_tree_view_column_new_with_attributes (_("Track"),
-                                                       renderer,
-                                                       "text", COLUMN_NUMBER,
+    gtk_tree_view_column_pack_start (column, renderer, FALSE);
+    gtk_tree_view_column_add_attribute (column, renderer, "text", COLUMN_NUMBER);
+    renderer = gtk_cell_renderer_pixbuf_new ();
+    g_object_set (renderer, "stock-size", GTK_ICON_SIZE_MENU, "xalign", 0.0, NULL);
+    gtk_tree_view_column_pack_start (column, renderer, TRUE);
+    gtk_tree_view_column_set_cell_data_func (column, renderer, number_cell_icon_data_cb, NULL, NULL);
+    gtk_tree_view_append_column (GTK_TREE_VIEW (track_listview), column);
+
+    title_renderer = gtk_cell_renderer_text_new ();
+    g_signal_connect (title_renderer, "edited", G_CALLBACK (on_cell_edited), GUINT_TO_POINTER (COLUMN_TITLE));
+    g_object_set (G_OBJECT (title_renderer), "editable", TRUE, NULL);
+    column = gtk_tree_view_column_new_with_attributes (_("Title"),
+                                                       title_renderer,
+                                                       "text", COLUMN_TITLE,
                                                        NULL);
     gtk_tree_view_column_set_resizable (column, TRUE);
-    gtk_tree_view_append_column (GTK_TREE_VIEW (track_listview), column);
-
-    column = gtk_tree_view_column_new ();
-    gtk_tree_view_column_set_title (column, _("Title"));
-    gtk_tree_view_column_set_resizable (column, TRUE);
     gtk_tree_view_column_set_expand (column, TRUE);
-    renderer = gtk_cell_renderer_pixbuf_new ();
-    gtk_tree_view_column_pack_start (column, renderer, FALSE);
-    gtk_tree_view_column_set_cell_data_func (column, renderer, title_cell_icon_data_cb, NULL, NULL);
-
-    renderer = gtk_cell_renderer_text_new ();
-    g_signal_connect (renderer, "edited", G_CALLBACK (on_cell_edited), GUINT_TO_POINTER (COLUMN_TITLE));
-    g_object_set (G_OBJECT (renderer), "editable", TRUE, NULL);
-    gtk_tree_view_column_pack_start (column, renderer, TRUE);
-    gtk_tree_view_column_add_attribute (column, renderer, "text", COLUMN_TITLE);
     gtk_tree_view_append_column (GTK_TREE_VIEW (track_listview), column);
 
-    renderer = gtk_cell_renderer_text_new ();
+    artist_renderer = gtk_cell_renderer_text_new ();
     column = gtk_tree_view_column_new_with_attributes (_("Artist"),
-                                                       renderer,
+                                                       artist_renderer,
                                                        "text", COLUMN_ARTIST,
                                                        NULL);
     gtk_tree_view_column_set_resizable (column, TRUE);
     gtk_tree_view_column_set_expand (column, TRUE);
-    g_signal_connect (renderer, "edited", G_CALLBACK (on_cell_edited), GUINT_TO_POINTER (COLUMN_ARTIST));
-    g_object_set (G_OBJECT (renderer), "editable", TRUE, NULL);
+    g_signal_connect (artist_renderer, "edited", G_CALLBACK (on_cell_edited), GUINT_TO_POINTER (COLUMN_ARTIST));
+    g_object_set (G_OBJECT (artist_renderer), "editable", TRUE, NULL);
     gtk_tree_view_append_column (GTK_TREE_VIEW (track_listview), column);
     
     renderer = gtk_cell_renderer_text_new ();
@@ -1180,6 +1262,11 @@ int main (int argc, char **argv)
     exit (1);
   }
 
+  sj_play_init ();
+
+  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (track_listview));
+  gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
+	  
   http_proxy_setup (gconf_client);
   /* TODO: port port basepath to baseuri */
   baseuri_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_BASEURI, NULL, TRUE, NULL), NULL);
